'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getFoo = exports.getSnapshotFile = undefined;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getSnapshotDir = function getSnapshotDir() {
  return _path2.default.join(process.cwd(), 'saul_snapshots');
};

// @t "gets file" getSnapshotFile(function tada() {}, 'foo') ~matches-snapshot
var getSnapshotFile = exports.getSnapshotFile = function getSnapshotFile(func, testDescription) {
  return func.name + '-' + _lodash2.default.kebabCase(testDescription) + '.snap';
};

// @t "gets foo" getFoo() ~matches-snapshot
var getFoo = exports.getFoo = function getFoo() {
  return 'bar';
};

/* istanbul ignore next */

exports.default = function (testDescription, func, argsArray, expected, test, _ref) {
  var getSpy = _ref.getSpy;

  test(testDescription, function () {
    if (!_fs2.default.existsSync(getSnapshotDir())) {
      _fs2.default.mkdirSync(getSnapshotDir());
    }

    var snapshotPath = _path2.default.join(getSnapshotDir(), getSnapshotFile(func, testDescription));

    var actualContent = JSON.stringify(func.apply(null, argsArray));

    if (!_fs2.default.existsSync(snapshotPath)) {
      _fs2.default.writeFileSync(snapshotPath, actualContent, {
        encoding: 'utf-8'
      });

      console.log('New snapshot recorded for ' + func.name + ': ' + testDescription);

      return;
    }

    var snapshotContent = void 0;

    snapshotContent = _fs2.default.readFileSync(snapshotPath, 'utf-8');

    // expect(snapshotContent).to.equal(actualContent);
    (0, _assert2.default)(snapshotContent === actualContent, 'Expected ' + snapshotContent + ' to equal ' + actualContent + '. If you want to rebuild the snapshot: rm ' + snapshotPath);
  });
};