'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.executeTestGroup = exports.executeTest = exports.groupTestsByFuncName = undefined;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _types = require('./types');

var _testcase = require('./testcase');

var _spies = require('./spies');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* istanbul ignore next */
var groupTestsByFuncName = exports.groupTestsByFuncName = function groupTestsByFuncName(tests) {
  return _lodash2.default.groupBy(tests, function (test) {
    if (!test.function) {
      throw new Error('Test ' + test.testDescription + ' does not hava a testable function. Did you export it?');
    }

    return test.function.name;
  });
};

// @t "executes with engine" executeTest({engine: () => 'result'}) ~deep-equals 'result'
// @t "throws on invalid" executeTest({engine: null}) ~throws Error
var executeTest = exports.executeTest = function executeTest(executableParams) {
  return executableParams.engine(executableParams.testDescription, executableParams.function, executableParams.args, executableParams.output, _testcase.test, {
    getSpy: _spies.getSpy
  });
};

// @t "passed suite context is called" executeTestGroup(null, null, spy('test')) ~expect-spy spy('test').calledOnce
var executeTestGroup = exports.executeTestGroup = function executeTestGroup(executableParams, funcName) {
  var testSuite = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _testcase.suite;

  testSuite(funcName, function () {
    return _lodash2.default.each(executableParams, executeTest);
  });
};

/* istanbul ignore next */
var execute = function execute(executableParams) {
  return (0, _lodash2.default)(executableParams).thru(groupTestsByFuncName).thru(function (groupedTests) {
    return _lodash2.default.forOwn(groupedTests, function (executableParams, funcName) {
      return executeTestGroup(executableParams, funcName);
    });
  }).commit();
};

exports.default = execute;