'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getEngineModule = exports.parseArgs = exports.getTestableFunction = undefined;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _types = require('./types');

var _config = require('./config');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _spies = require('./spies');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

require('babel-register');

// @t "with normal export"    getTestableFunction('foo', 'bar', () => ({'bar': 'baz'})) ~equals 'baz'
// @t "with nested export"    getTestableFunction('foo', 'bar.good', () => ({'bar': { 'good': 'sogood' }})) ~equals 'sogood'
var getTestableFunction = exports.getTestableFunction = function getTestableFunction(filepath, funcName) {
  var requireFile = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : require;
  var get = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : _lodash2.default.get;

  var module = requireFile(filepath);
  return get(module, funcName);
};

var getArgsContext = function getArgsContext() {
  return {
    spy: _spies.generateSpy
  };
};

function parseArgsFn(argsString) {
  try {
    // eslint-disable-next-line no-eval
    return eval('\n      const spy = this.spy;\n      [' + argsString + ']\n    ');
  } catch (e) {
    throw new Error('Possible syntax error in your args: ' + argsString);
  }
}

// @t "parses correctly"            parseArgs("0") ~deep-equals [0]
// @t "parses multiple values"      parseArgs("12, 15") ~deep-equals [12, 15]
// @t "accepts spies"               parseArgs("spy('foo'), 15") ~throws false
// @t "throws on invalid spies"     parseArgs("spyz('0'), 15") ~throws true
var parseArgs = exports.parseArgs = parseArgsFn.bind(getArgsContext());

var getEngineModule = exports.getEngineModule = function getEngineModule(engineName) {
  var requireFile = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : require;

  try {
    var userModule = _path2.default.join(process.cwd(), (0, _config.getConfig)().customEnginesDir, engineName + '.js');

    if (_fs2.default.statSync(userModule) && requireFile(userModule).default) {
      return requireFile(userModule).default;
    }
  } catch (e) {
    return requireFile('./engines/' + engineName + '.js').default;
  }
};

exports.default = function (filepath, testParams) {
  return {
    function: getTestableFunction(filepath, testParams.funcName),
    testDescription: testParams.testDescription,
    args: parseArgs(testParams.args),
    engine: getEngineModule(testParams.engineName),
    output: testParams.output
  };
};